--------------------NOTICE-------------------------------*
-- Software ID: w32utils.ew
-- Version:     0.54 "Bleeding Edge" 07/Oct/200
-- Copyright:   (c) 2000 David Cuny and friends
--              All rights reserved.
-- Licence:            
-- This software is provided 'as-is', without any express or implied warranty.
-- In no event will the authors be held liable for any damages arising from
-- the use of this software.
--
-- Permission is granted to anyone to use this software for any purpose,
-- including commercial applications, and to alter it and redistribute it
-- freely, subject to the following restrictictions:
-- 1. The origin of this software must not be misrepresented; you must not
--    claim that you wrote the original software.
-- 2. If you use this software in a product, acknowedgement in the product's
--    documenation and binary are required.
-- 3. Altered source versions, and works substantially derived from the it,
--    must...
--   a) be plainly be marked as such,
--   b) not be misrepresented as the original software,
--   c) include this notice, unaltered.
--------------------End of NOTICE------------------------*
--/topic Utilities
--/info
--Miscellaneous "utility" type routihes.
--These are miscellaneous utility functions and procedures that can be used
--with many Euphoria projects.

--/topic Utilities
--/const True 
--/desc Defines the Euphoria TRUE value
--/const False
--/desc Defines the Euphoria FALSE value

global constant
    False   = (1 = 0),
    True    = (1 = 1)

--/topic Utilities
--/func findKey( key, list )
--/desc Find key in list.
--/ret Index of sublist with matching key.
--
-- Example:
--
--/code 
--      -- find a value from a list
--      integer at
--      at = findKey( name, nameList )
--/endcode

global function findKey( object key, sequence list )
    for i = 1 to length( list ) do
        if equal( list[i][1], key ) then
            return i
        end if
    end for
    return 0

end function


--/topic Utilities
--/func findAltKey( key, list )
--/desc Find alternate key in list.
--/ret Index of sublist with matching key in a specific column.
--
-- Example:
--
--/code 
--      -- find a value from column 2 in a list
--      integer at
--      at = findAltKey( 2, name, nameList )
--/endcode

global function findAltKey( integer col, object key, sequence list )
    for i = 1 to length( list ) do
        if equal( list[i][col], key ) then
            return i
        end if
    end for
    return 0

end function


----------------------------------------------------------------------------
--/topic Utilities
--/func removeIndex( integer index, sequence list )
--/desc Remove the element at position /i index from the sequence /i list
--/ret The /i list without the element. 
--
--If /i index is greater than the list length, the list is returned intact.
-- If /i index is less than 1, it represents the end of the list PLUS 
-- the index. So an index of '-1' means the second-last element.
--
-- Example:
--
--/code
--      -- Remove the 4th element from the Name List
--      nameList = removeIndex( 4, nameList )
--      -- Remove the last element.
--      nameList = removeIndex( 0, nameList )
--/endcode

global function removeIndex( integer index, sequence list )
    -- Sanity check for non-existant elements.
    if index > length(list) then
        return list
    end if
    
    if index < 1 then
        index = length(list) + index
        if index < 1 then
            return list
        end if
    end if

    return list[1 .. index-1] & list[index+1 .. length(list)]
end function

--/topic Utilities
--/func insertIndex( integer index, sequence list, object Data )
--/desc Inserts /i Data at position /i index in the sequence /i list
--/ret The /i list with the new element.
--If /i index is zero or less, the /i Data is added to the front of the /i list
-- and if /i index is passed the end of the list, /i Data is added to the
-- end of the /i list.
--
-- Example:
--
--/code
--      -- Insert a new name into position 4 of the list
--      nameList = insertIndex( 4, nameList, "Michael Palffy" )
--/endcode

----------------------------------------------------------------------------
global function insertIndex(integer index, sequence list, object pData)
    -- Method by Michael Palffy
    integer len

    len = length(list) + 1

    if index < 1 then
        return prepend(list, pData)
    elsif index >= len then
        return append(list, pData)
    -- Test for the smallest # of elements to move.
    elsif index < (len - index) then
        list = 0 & list  -- reserve space at the front
        list[1 .. index-1] = list[2 .. index] -- move front portion down
    else
        list &= 0    -- reserve some space at the end
        list[index+1 .. len] = list[index .. len-1] -- move back portion up
    end if
    list[index] = pData -- assign the new value

    return list
end function

--/topic Utilities
--/func rotate_elements( sequence list, sequence index )
--/desc moves elements about in /i list according to the /i index set.
--/ret sequence: The updated list
--For each element of /i list identified in the /i index set, is moved
--
--
--Example:        
--      -- Swap the 3rd and 4th element.
--      theList = rotate_elements(theList, {3,4})    
--      -- Move 2nd to 8th, 4th to 2nd, 6th to 4th, and 8th to 6th.
--      theList = rotate_elements(theList, {2,4,6,8})
global function rotate_elements(sequence list, sequence index)
    object a
    a = list[index[1]]
    for i = 2 to length(index) do
        list[index[i-1]] = list[index[i]]
    end for
    list[index[length(index)]] = a        
    return list
end function

--/topic Utilities
--/func iff (atom test, object ifTrue, object ifFalse)
--/desc Used to embed an 'if' test inside an expression.
--/ret If /i test is /b true then /i ifTrue is returned otherwise /i ifFalse is returned.
--
--Example
--/code
--         msg = sprintf("%s: %s", {
--                      iff(ErrType = 'E', "Fatal error", "Warning"),
--                      errortext } )
--/endcode

----------------------------------------------------------------------------
global function iff( atom test, object ifTrue, object ifFalse )
    -- returns ifTrue if flag is true, else returns ifFalse
    -- ex:  ? iff( 1, "this", "that" )
    --      --> "this"
    if test then
        return ifTrue
    else
        return ifFalse
    end if

end function

--/topic Utilities
--/func removeItem( object item, sequence list)
--/desc Removes /i item from the /i list, if it is in the list.
--/ret The /i list without the item.
--
--Example
--/code
--     -- Remove the name 'fred' from the list.
--     nameList = removeItem("fred", nameList)
--/endcode
----------------------------------------------------------------------------
global function removeItem( object item, sequence list )
    -- Removes the item from the list if item is in the list.
    item = find(item,  list )
    if item > 0 then
        return removeIndex(item, list)
    else
        return list
    end if
end function


--/topic Utilities
--/func removeKey( object item, sequence list)
--/desc Removes /i keyed item from the /i list, if it is in the list.
--/ret The /i list without the item.
--The difference between this and /removeItem is that in this function,
-- /i list is a list of sequences and /i item is found if it matches the
-- first element in the sequences. /n
--Example
--/code
--     -- Remove the tracked resource.
--     ResourceList = removeKey(hWnd, ResourceList)
--     -- Add new tracked item.
--     ResourceList = insertIndex(0, ResourceList, {hWnd, 0, TagData})
--
--/endcode
----------------------------------------------------------------------------
global function removeKey( object item, sequence list )
    -- Removes the keyed item from the list if item is in the list.
    item = findKey(item,  list )
    if item > 0 then
        return removeIndex(item, list)
    else
        return list
    end if
end function



--/topic Utilities
--/func DIMArray(sequence Dimension, object InitValue)
--/desc Creates an array (sequence) with the dimensions specified.
--/ret SEQUENCE: An initialised array. However if it fails an INTEGER is returned.
-- The /i Dimension parameter is a list of lengths for each dimension. For a 3 dimension 
-- array you would use a list of 3 lengths. Each length must be an integer with a value 
-- greater than zero. /n
-- The /i InitValue is the value to set for each element. It can be any Euphoria object.
--
-- Note that if the function fails, it returns an integer. 
--<ul>
--/li 0: The /i Dimension sequence was empty
--/li 1 - length(Dimension): The index of the length that caused a problem.
--</ul>
--
--Example
--/code
--    object Space3D
--    Space3D = DIMArray( {3, 7, 4}, 0)
--    if integer(Space3D) then
--        ErrAbort(sprintf("All dimensions must be positive integers; dimension %d is not.", Space3D))
--    end if
--
--/endcode
----------------------------------------------------------------------------
global function DIMArray( sequence Dimension, object InitValue )
    sequence lNewArray

    if length(Dimension) = 0 then
        return 0
    end if
    lNewArray = InitValue
    for i = length(Dimension) to 1 by -1 do
       if atom(Dimension[i]) and Dimension[i] > 0 then
           lNewArray = repeat(lNewArray, Dimension[i])
       else
           return i
       end if
    end for
    return lNewArray
end function
