-- tracker.e 
-- by Derek Parnell, December 2002.
--                                     
-- The purpose of this library is to enable run-time tracing
-- of a program without using the Euphoria trace facility.

-- This contains the following routines...
--
--  Track( object x)
--      This causes 'x' to be written out to a tracking file.
--      By default, this file is called "tracking.txt". The
--      name can be overridden by the calling program using
--      the 'SetTrackingFile()' routine or by a command-line
--      option at startup. The command line option is 
--        --track=<filename> where "<filename>" is replaced 
--      with a file name. If the file name is omitted, then
--      no tracking is done.
--      eg.  exw myprog --track=myprog.trk
--              This sends tracking info to the file "myprog.trk"
--           exw myprog --track=
--              This stops tracking for this run.

--  WithTracking( integer flag)
--      This routine allows the calling program to turn tracking
--      on or off as required. If the 'flag' is zero then tracking
--      stops, otherwise it continues. Note that if tracking was
--      turned off by the command-line option, this routine can
--      turn it back on again.

--  ClearTracking()
--      This clears the tracking file. All existing data in it is lost.
--      It writes the current date and time, and the command line
--      parameters to the file.

--  SetTrackingFile(sequence filename)
--      This routine allows the calling program to set the file name to
--      use for the next Track() call. It is possible to have multiple
--      tracking files being used concurrently.

--  GetTracking()
--      This function returns the value of a number of internal
--      variables.
--      It returns a sequence that holds ...
--      { IsTrackingFlag, TrackingLineCount, TrackingFileName }
--      where ...
--          IsTrackingFlag: 0 if not tracking otherwise 1.
--          TrackingLineCount: The number of lines output so far.
--          TrackingFileName: The name of the file that will receive 
--                              the next tracking line.

-- You would use this library by first including it, then
-- placing calls to the 'Tracking()' routine in the areas
-- of you code that were suspect. At anytime during the running
-- of you program you can view the contents of the tracking file
-- to see which of your Tracking() calls have been run and which
-- order they were invoked in. The parameter passed to Tracking
-- can help you identify the specific Tracking call in your source
-- file.

-- eg.
--     include tracker.e
--      . . .
--     procedure xyz(integer a, sequence b)
--        Tracking( {"xyz", a, b })
--        . . .
--        c = APICall("GetClientRect", {mywindow})
--        Tracking(c)
--        . . .
--     end procedure

without trace
without profile
without type_check
without warning

sequence vTrackFile         -- This holds the name of the tracking
                            -- file.

integer  vTracking          -- A flag that indicates whether we are
                            -- tracking or not.

integer  vCnt               -- This counts the lines written out
                            -- to the tracking file.

integer  ri_ClearTracking   -- This will hold a routine_id. It 
                            -- is required due to RDS's restrictive
                            -- philosophy about forward references.
                         
-- Local Routines --      
------------------------------------------------------
function ReplaceElement(sequence pFind, sequence pReplacement, sequence pTarget)
------------------------------------------------------
    integer lPosn
    integer lFindLength
    
    lPosn = match(pFind, pReplacement)
    if lPosn != 0 then
        -- Must do a complex replacement.
        lPosn = -1
        while find(lPosn, pFind) or find(lPosn, pReplacement) or find(lPosn, pTarget) do
            lPosn -= 1
        end while
        pTarget = ReplaceElement(pFind, {lPosn}, pTarget)
        pTarget = ReplaceElement({lPosn}, pReplacement, pTarget)
        
    else               
        lPosn = match(pFind, pTarget)
        while lPosn != 0 do
            pTarget = pTarget[1..lPosn-1] &
                      pReplacement &
                      pTarget[lPosn+length(pFind) .. length(pTarget)]
            lPosn = match(pFind, pTarget)
        end while
    end if
    
    return pTarget
end function

------------------------------------------------------
function String(sequence pData)
-- Tests whether or not a sequence is a printable string.
-- Returns 0 if not, 1 if it is.
------------------------------------------------------
    for i = 1 to length(pData) do
        if not integer(pData[i]) or 
            pData[i] > '~' or 
            (pData[i] < ' ' and not find(pData[i], {9,10,13,0})) then
            return 0
        end if
    end for
    return 1                        
end function

------------------------------------------------------
function Format(object pData)
-- Formats an object so that it can be printed out.
-- Returns a sequence that can be directly output
--   to a user interface device.
------------------------------------------------------
    sequence lData
    integer lDotPos
    
    if integer(pData) then
        lData = sprintf("%d", pData)
        
    elsif atom(pData) then
        lData = sprintf("%20.20f", pData)
        lDotPos = find('.', lData)
        if lDotPos != 0 then
            -- Remove trailing zeroes.
            for i = length(lData) to lDotPos by -1 do
                if lData[i] != '0' then
                    if i != lDotPos then
                        lData = lData[1..i]
                    else
                        lData = lData[1..i+1]                        
                    end if
                    exit
                end if
            end for
        end if
    elsif String(pData) then
        pData = ReplaceElement({9}, "\\t", pData)
        pData = ReplaceElement({10}, "\\n", pData)
        pData = ReplaceElement({13}, "\\r", pData)
        pData = ReplaceElement({0}, "\\0", pData)
        lData = sprintf("\"%s\"", {pData}) 
    else
        lData = "{"
        for i = 1 to length(pData) do
            
            lData &= Format(pData[i])
            
            if i < length(pData) then
                lData &= ','
            end if

        end for
        lData &= '}'
    end if
    
    return lData
end function

------------------------------------------------------
procedure ftt()
-- Performs the First Time Through actions. This is 
-- only run once, when the file is first included.
------------------------------------------------------
    sequence lCmdLine
    
    -- sSet default values.
    vTrackFile = "tracker.txt"                        
    vTracking = 1
    vCnt = 0
                           
    -- See if the defaults are overridden at run-time.
    lCmdLine = command_line()       
    for i = 3 to length(lCmdLine) do            
        if length(lCmdLine[i]) >= 8  then
            if match("--track=", lCmdLine[i]) = 1 then
                vTrackFile = lCmdLine[i][9..length(lCmdLine[i])]
                exit
            end if
        end if
    end for        
    
    -- The STUPID rule about 'no forward references' means that if I 
    -- want to have neat code, I also have to use routine_id here. Sorry.
    call_proc(ri_ClearTracking,{})
end procedure    
             

-- Global Routines --
------------------------------------------------------
global procedure ClearTracking()
------------------------------------------------------
    integer lFH
    sequence lNow
                    
    lNow = date()
    lNow = sprintf("%4d-%02d-%02d.%02d:%02d:%02d Tracking Started for\n%s\n", {
                        lNow[1]+1900, lNow[2], lNow[3],
                        lNow[4], lNow[5], lNow[6], Format(command_line())})
    lFH = open(vTrackFile, "w")
    if lFH > -1 then                                             
        puts(lFH, lNow)
        close(lFH)
        vTracking = 1
    else
        vTracking = 0    
    end if    
end procedure                                 
-- Load variable that I am forced to use if I want forward referencing.
ri_ClearTracking = routine_id("ClearTracking")

------------------------------------------------------
global procedure WithTracking(integer pFlag)
------------------------------------------------------
    if pFlag = 0 then
        vTracking = 0
    else
        vTracking = 1
    end if
end procedure

------------------------------------------------------
global procedure SetTrackingFile(sequence pNewName)
------------------------------------------------------
    vTrackFile = pNewName
end procedure

------------------------------------------------------
global function GetTracking()
------------------------------------------------------
    return {vTracking, vCnt, vTrackFile}
end function

------------------------------------------------------
global procedure Track(object pData)
------------------------------------------------------
    sequence lData
    integer lFH
    
    if vTracking = 0 then
        return
    end if
                 
                                       
    vCnt += 1
    lData = sprintf("%6d: %s\n", {vCnt, Format(pData)})
    
    lFH = open(vTrackFile, "a")
    if lFH > -1 then
        puts(lFH, lData)
        close(lFH)
    else
        vTracking = 0    
    end if    

end procedure

------------------------------------------------------
-- Do the First Time Through stuff.
------------------------------------------------------
ftt()
                                        
Track("Derek Parnell" & 0)